import LinkifyIt from 'linkify-it'
import UrlParse from 'url-parse'

const linkify = LinkifyIt()

const embedWhiteListAndCode = {
  'www.youtube.com': {
    type: 'youtube',
    code: '<iframe class="youtube-player" type="text/html" src="" frameborder="0"  allowfullscreen></iframe>'
  },
}

export const htmlEntities = (str) => {
  return String(str).replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;')
}

export const getUrlParameter = (sParam) => {
  let sPageURL = window.location.search.substring(1),
      sURLVariables = sPageURL.split('&'),
      sParameterName = [];

  for (let i = 0; i < sURLVariables.length; i++) {
    sParameterName = sURLVariables[i].split('=');
    if (sParameterName[0] === sParam)
      return sParameterName[1] === undefined ? '' : decodeURIComponent(sParameterName[1]);
  }
  return ''
}

export const parseText = (str, len) => {
  let tmp = '', count = 0;
  for (let i = 0; i < str.length; i++) {
    if (str[i].match(/[\u4e00-\u9fa5]/g)) tmp += str[i], count += 2
    else if (str[i].match(/[\u0800-\u4e00]/g)) tmp += str[i], count += 2
    else if (str[i].match(/[\uff00-\uffff]/g)) tmp += str[i], count += 2
    else tmp += str[i], count++

    if (count === len) break
    else if (count > len)
      tmp = tmp.substr(0, tmp.length - 1)
  }
  return tmp
}

export const parseUser = (address, meta) => {
  if (meta && meta.name) {
    return meta.name
  }
  return address.replace(/^(0x.{4}).+(.{4})$/, '$1…$2')
}

export const formatPttDateTime = ts => {
  return (''+new Date(ts)).substr(0,24)
}

const createEmbedObject = (url) => {
  const parsedUrl = new UrlParse(url)
  const ret = {
    allowed: true,
    element: null
  }

  // https://www.npmjs.com/package/url-parse
  // host: Host name with port number.
  // hostname: Host name without port number.
  const embedMap = embedWhiteListAndCode[parsedUrl.hostname]
  if (embedMap === undefined) {
    // Not in white list
    ret.allowed = false
    return ret
  }

  const elParent1 = $('<div class="richcontent"></div>')
  const elParent2 = $('<div class="resize-container"></div>')
  const elParent3 = $('<div class="resize-content"></div>')
  const el = $(embedMap.code)

  if (embedMap.type === 'youtube') {
    const processedUrl = `https://www.youtube.com/embed/${parsedUrl.query.replace('?v=', '')}`
    el.attr('src', processedUrl)
    elParent3.html(el)
    elParent2.html(elParent3)
    elParent1.html(elParent2)
  }

  ret.element = elParent1[0]

  return ret
}

export const parseContent = (content, loc) => {
  let matches = linkify.match(content)
  let result = []

  if (matches) {
    let last = 0
    matches.forEach(match => {
      if (last < match.index) {
        result.push(document.createTextNode(content.slice(last, match.index)));
      }

      const el = document.createElement('a')
      el.target = '_blank'
      el.href = match.url
      el.textContent = match.text
      result.push(el)

      // Handle embedded links, only in post not reply
      let embedObject = loc === 'post' ? createEmbedObject(match.url) : null
      if (embedObject && embedObject.allowed) {
        result.push(embedObject.element)
      }

      last = match.lastIndex
    });
    if (last < content.length) {
      result.push(document.createTextNode(content.slice(last)))
    }
  } else {
    result.push(document.createTextNode(content))
  }

  return result
}

export const awaitTx = maybePromiseEvent => {
  let fulfilled = false
  if (!maybePromiseEvent.on) {
    // not a promievent
    return maybePromiseEvent
  }
  return new Promise((resolve, reject) => {
    maybePromiseEvent.on('confirmation', (no, receipt) => {
      if (fulfilled) return
      fulfilled = true
      resolve(receipt)
    })
    .catch(err => {
      if (fulfilled) return
      fulfilled = true
      reject(err)
    })
  })
}

export function web3ErrorToString(err) {
  const msg = err.message
  if (!err.message) {
    return err.toString()
  }
  const grp = msg.match(/^Node error: (.+)/)
  try {
    // TODO: add proper translation from code property
    return JSON.parse(grp[1]).message

  } catch (_) {
    return msg
  }
}
